@@ -1,15 +1,25 @@
 local files     = require 'files'
 local lang      = require 'language'
 local util      = require 'utility'
-local sp        = require 'bee.subprocess'
 local guide     = require "parser.guide"
 local converter = require 'proto.converter'
+local autoreq   = require 'core.completion.auto-require'
+local rpath     = require 'workspace.require-path'
+local furi      = require 'file-uri'
+local vm        = require 'vm'
 
+---@param uri  uri
+---@param row  integer
+---@param mode string
+---@param code string
 local function checkDisableByLuaDocExits(uri, row, mode, code)
     if row < 0 then
         return nil
     end
     local state = files.getState(uri)
+    if not state then
+        return nil
+    end
     local lines = state.lines
     if state.ast.docs and lines then
         return guide.eachSourceBetween(
@@ -39,7 +49,7 @@ local function checkDisableByLuaDocExits(uri, row, mode, code)
     return nil
 end
 
-local function checkDisableByLuaDocInsert(uri, row, mode, code)
+local function checkDisableByLuaDocInsert(_uri, row, mode, code)
     return {
         start   = guide.positionOf(row, 0),
         finish  = guide.positionOf(row, 0),
@@ -124,9 +134,12 @@ local function changeVersion(uri, version, results)
 end
 
 local function solveUndefinedGlobal(uri, diag, results)
-    local ast    = files.getState(uri)
-    local start  = converter.unpackRange(uri, diag.range)
-    guide.eachSourceContain(ast.ast, start, function (source)
+    local state = files.getState(uri)
+    if not state then
+        return
+    end
+    local start = converter.unpackRange(state, diag.range)
+    guide.eachSourceContain(state.ast, start, function (source)
         if source.type ~= 'getglobal' then
             return
         end
@@ -143,9 +156,12 @@ local function solveUndefinedGlobal(uri, diag, results)
 end
 
 local function solveLowercaseGlobal(uri, diag, results)
-    local ast    = files.getState(uri)
-    local start  = converter.unpackRange(uri, diag.range)
-    guide.eachSourceContain(ast.ast, start, function (source)
+    local state = files.getState(uri)
+    if not state then
+        return
+    end
+    local start = converter.unpackRange(state, diag.range)
+    guide.eachSourceContain(state.ast, start, function (source)
         if source.type ~= 'setglobal' then
             return
         end
@@ -156,10 +172,13 @@ local function solveLowercaseGlobal(uri, diag, results)
 end
 
 local function findSyntax(uri, diag)
-    local ast = files.getState(uri)
-    for _, err in ipairs(ast.errs) do
+    local state = files.getState(uri)
+    if not state then
+        return
+    end
+    for _, err in ipairs(state.errs) do
         if err.type:lower():gsub('_', '-') == diag.code then
-            local range = converter.packRange(uri, err.start, err.finish)
+            local range = converter.packRange(state, err.start, err.finish)
             if util.equal(range, diag.range) then
                 return err
             end
@@ -221,7 +240,7 @@ local function solveSyntaxByFix(uri, err, results)
     }
 end
 
-local function solveSyntaxUnicodeName(uri, err, results)
+local function solveSyntaxUnicodeName(uri, _err, results)
     results[#results+1] = {
         title   = lang.script('ACTION_RUNTIME_UNICODE_NAME'),
         kind    = 'quickfix',
@@ -260,7 +279,11 @@ local function solveSyntax(uri, diag, results)
 end
 
 local function solveNewlineCall(uri, diag, results)
-    local start = converter.unpackRange(uri, diag.range)
+    local state = files.getState(uri)
+    if not state then
+        return
+    end
+    local start = converter.unpackRange(state, diag.range)
     results[#results+1] = {
         title = lang.script.ACTION_ADD_SEMICOLON,
         kind = 'quickfix',
@@ -296,7 +319,7 @@ local function solveAmbiguity1(uri, diag, results)
     }
 end
 
-local function solveTrailingSpace(uri, diag, results)
+local function solveTrailingSpace(uri, _diag, results)
     results[#results+1] = {
         title = lang.script.ACTION_REMOVE_SPACE,
         kind = 'quickfix',
@@ -317,7 +340,7 @@ local function solveAwaitInSync(uri, diag, results)
     if not state then
         return
     end
-    local start, finish = converter.unpackRange(uri, diag.range)
+    local start, finish = converter.unpackRange(state, diag.range)
     local parentFunction
     guide.eachSourceType(state.ast, 'function', function (source)
         if source.start > finish
@@ -333,6 +356,8 @@ local function solveAwaitInSync(uri, diag, results)
     end
     local row = guide.rowColOf(parentFunction.start)
     local pos = guide.positionOf(row, 0)
+    local offset = guide.positionToOffset(state, pos + 1)
+    local space = state.lua:match('[ \t]*', offset)
     results[#results+1] = {
         title = lang.script.ACTION_MARK_ASYNC,
         kind = 'quickfix',
@@ -342,7 +367,7 @@ local function solveAwaitInSync(uri, diag, results)
                     {
                         start   = pos,
                         finish  = pos,
-                        newText = '---@async\n',
+                        newText = space .. '---@async\n',
                     }
                 }
             }
@@ -350,6 +375,55 @@ local function solveAwaitInSync(uri, diag, results)
     }
 end
 
+local function solveSpell(uri, diag, results)
+    local state = files.getState(uri)
+    if not state then
+        return
+    end
+    local spell = require 'provider.spell'
+    local word = diag.data
+    if word == nil then
+        return
+    end
+
+    results[#results+1] = {
+        title   = lang.script('ACTION_ADD_DICT', word),
+        kind    = 'quickfix',
+        command = {
+            title     = lang.script.COMMAND_ADD_DICT,
+            command   = 'lua.setConfig',
+            arguments = {
+                {
+                    key    = 'Lua.spell.dict',
+                    action = 'add',
+                    value  = word,
+                    uri    = uri,
+                }
+            }
+        }
+    }
+
+    local suggests = spell.getSpellSuggest(word)
+    for _, suggest in ipairs(suggests) do
+        results[#results+1] = {
+            title = suggest,
+            kind = 'quickfix',
+            edit = {
+                changes = {
+                    [uri] = {
+                        {
+                            start   = converter.unpackPosition(state, diag.range.start),
+                            finish  = converter.unpackPosition(state, diag.range["end"]),
+                            newText = suggest
+                        }
+                    }
+                }
+            }
+        }
+    end
+
+end
+
 local function solveDiagnostic(uri, diag, start, results)
     if diag.source == lang.script.DIAG_SYNTAX_CHECK then
         solveSyntax(uri, diag, results)
@@ -370,6 +444,8 @@ local function solveDiagnostic(uri, diag, start, results)
         solveTrailingSpace(uri, diag, results)
     elseif diag.code == 'await-in-sync' then
         solveAwaitInSync(uri, diag, results)
+    elseif diag.code == 'spell-check' then
+        solveSpell(uri, diag, results)
     end
     disableDiagnostic(uri, diag.code, start, results)
 end
@@ -386,7 +462,7 @@ end
 local function checkSwapParams(results, uri, start, finish)
     local state = files.getState(uri)
     local text  = files.getText(uri)
-    if not state then
+    if not state or not text then
         return
     end
     local args = {}
@@ -414,7 +490,7 @@ local function checkSwapParams(results, uri, start, finish)
                 )
             elseif source.type == 'funcargs' then
                 local var = source.parent.parent
-                if guide.isSet(var) then
+                if guide.isAssign(var) then
                     if var.type == 'tablefield' then
                         var = var.field
                     end
@@ -554,26 +630,37 @@ end
 local function checkJsonToLua(results, uri, start, finish)
     local text         = files.getText(uri)
     local state        = files.getState(uri)
+    if not state or not text then
+        return
+    end
     local startOffset  = guide.positionToOffset(state, start)
     local finishOffset = guide.positionToOffset(state, finish)
-    local jsonStart    = text:match('()[%{%[]', startOffset + 1)
+    local jsonStart = text:match('()["%{%[]', startOffset + 1)
     if not jsonStart then
         return
     end
-    local jsonFinish
+    local jsonFinish, finishChar
     for i = math.min(finishOffset, #text), jsonStart + 1, -1 do
         local char = text:sub(i, i)
         if char == ']'
         or char == '}' then
             jsonFinish = i
+            finishChar = char
             break
         end
     end
     if not jsonFinish then
         return
     end
-    if not text:sub(jsonStart, jsonFinish):find '"%s*%:' then
-        return
+    if finishChar == '}' then
+        if not text:sub(jsonStart, jsonFinish):find '"%s*%:' then
+            return
+        end
+    end
+    if finishChar == ']' then
+        if not text:sub(jsonStart, jsonFinish):find ',' then
+            return
+        end
     end
     results[#results+1] = {
         title = lang.script.ACTION_JSON_TO_LUA,
@@ -592,6 +679,55 @@ local function checkJsonToLua(results, uri, start, finish)
     }
 end
 
+local function findRequireTargets(visiblePaths)
+    local targets = {}
+    for _, visible in ipairs(visiblePaths) do
+        targets[#targets+1] = visible.name
+    end
+    return targets
+end
+
+local function checkMissingRequire(results, uri, start, finish)
+    local state = files.getState(uri)
+    local text  = files.getText(uri)
+    if not state or not text then
+        return
+    end
+
+    local function addRequires(global, endpos)
+        autoreq.check(state, global, endpos, function (moduleFile, _stemname, _targetSource, fullKeyPath)
+            local visiblePaths = rpath.getVisiblePath(uri, furi.decode(moduleFile))
+            if not visiblePaths or #visiblePaths == 0 then return end
+
+            for _, target in ipairs(findRequireTargets(visiblePaths)) do
+                results[#results+1] = {
+                    title = lang.script('ACTION_AUTOREQUIRE', target .. (fullKeyPath or ''), global),
+                    kind = 'refactor.rewrite',
+                    command = {
+                        title     = 'autoRequire',
+                        command   = 'lua.autoRequire',
+                        arguments = {
+                            {
+                                uri         = guide.getUri(state.ast),
+                                target      = moduleFile,
+                                name        = global,
+                                requireName = target,
+                                fullKeyPath = fullKeyPath,
+                            },
+                        },
+                    }
+                }
+            end
+        end)
+    end
+
+    guide.eachSourceBetween(state.ast, start, finish, function (source)
+        if vm.isUndefinedGlobal(source) then
+            addRequires(source[1], source.finish)
+        end
+    end)
+end
+
 return function (uri, start, finish, diagnostics)
     local ast = files.getState(uri)
     if not ast then
@@ -604,6 +740,7 @@ return function (uri, start, finish, diagnostics)
     checkSwapParams(results, uri, start, finish)
     --checkExtractAsFunction(results, uri, start, finish)
     checkJsonToLua(results, uri, start, finish)
+    checkMissingRequire(results, uri, start, finish)
 
     return results
 end
